package br.UFSC.GRIMA.simulator;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.RenderingHints;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;

import javax.swing.BorderFactory;
import javax.swing.JPanel;
import javax.swing.Timer;


public class ToolPanel extends JPanel {
	// ============================================== fields

	private static final long serialVersionUID = 1L;
	private Point initialPoint = new Point(0, 0);
	private Point lastPoint = new Point(0, 0);
	private Graphics2D g2d;
	private int pointIndex;
	private boolean isDrilling = false;
	boolean pass = true;
	private double velocity = 100;

	public double getVelocity() {
		return velocity;
	}

	public void setVelocity(double velocity) {
		this.velocity = velocity;
		tool.setVelocity(velocity);
	}

	// ... Instance variables representing the ball.
	private Tool tool = new Tool(0, 0, 2, 3);

	private ArrayList<Point> arrayListPoints;

	private ToolPath toolPath;

	// ... Instance variables for the animiation
	private int m_interval = 35; // Milliseconds between updates.
	private Timer m_timer; // Timer fires to anmimate one step.

	int blockX;
	int blockY;

	int lineIndex = 0;

	// ========================================================== constructor
	/** Set panel size and creates timer. */
	public ToolPanel(Rectangle r, ArrayList<Point> arrayListPoints) {
		setPreferredSize(new Dimension(500, 500));
		setBorder(BorderFactory.createLineBorder(Color.BLACK));
		m_timer = new Timer(m_interval, new TimerAction());

		blockX = (int) r.getWidth();
		blockY = (int) r.getHeight();

		this.arrayListPoints = arrayListPoints;

		
		toolPath = new ToolPath(arrayListPoints);

	}

	// ========================================================= setAnimation
	/**
	 * Turn animation on or off.
	 * 
	 * @param turnOnOff
	 *            Specifies state of animation.
	 */
	public void setAnimation(boolean turnOnOff) {
		if (turnOnOff) {
			m_timer.start(); // start animation by starting the timer.
		} else {
			m_timer.stop(); // stop timer
		}
	}

	// ======================================================= paintComponent
	public void paintComponent(Graphics g) {

		g2d = (Graphics2D) g;

		setGraphicsOptions();

		setAxis();

		super.paintComponent(g); // Paint background, border

		drawBlock();

		drawHoles();

		drawGrade();

		drawWrintings();
		
		if(isDrilling){
			drawMesage();
		}

		drawLines();

		drawAxis();

		drawTool();

	}

	private void drawHoles() {
		g2d.setColor(new Color(205, 133, 63));

		for (int i = 0; i < pointIndex; i++) {
			if (i > 0) {
				Point p = (Point) arrayListPoints.get(i - 1);
				if (i != 1) {
					g2d.fillOval((int) Math
							.round(p.getY())
							- tool.getDiameter() / 2,(int) Math.round(p.getX())
							- tool.getDiameter() / 2, tool.getDiameter(), tool
							.getDiameter());
				}
			}

		}
		g2d.setColor(Color.BLACK);
	}

	private void drawAxis() {
		float dash1[] = { 5.0f, 0f };
		g2d.setStroke(new BasicStroke(3.0f, BasicStroke.CAP_BUTT,
				BasicStroke.JOIN_MITER, 10.0f, dash1, 0.0f));

		g2d.drawLine( (int) Math.round(tool.getY()),0,  (int) Math
				.round(tool.getY()),500);
		g2d.drawLine(0,(int) Math.round(tool.getX()), 500, (int) Math.round(tool
				.getX()));

		g2d.setStroke(new BasicStroke());
	}

	public void setGraphicsOptions() {
		g2d.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING,
				RenderingHints.VALUE_TEXT_ANTIALIAS_ON);
		g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
				RenderingHints.VALUE_ANTIALIAS_ON);
	}

	private void drawGrade() {

		float dash1[] = { 5.0f, 2.5f };
		g2d.setStroke(new BasicStroke(1.0f, BasicStroke.CAP_BUTT,
				BasicStroke.JOIN_MITER, 10.0f, dash1, 0.0f));

		g2d.setColor(Color.GRAY);
		for (int i = 0; i < 10; i++) {
			g2d.drawLine(i * 50, 0, i * 50, 500);
		}
		for (int i = 0; i < 10; i++) {
			g2d.drawLine(0, i * 50, 500, i * 50);
		}
		g2d.setColor(Color.BLACK);

		g2d.setStroke(new BasicStroke());
	}

	private void setAxis() {
		g2d.translate(0, 500);
		g2d.rotate(-Math.PI / 2);
	}

	private void drawTool() {
		tool.draw(g2d); // Draw the ball.

	}

	private void drawBlock() {
		g2d.setColor(Color.LIGHT_GRAY);
		g2d.fillRect(0, 0, blockY, blockX);
	}

	private void drawLines() {

		float dash1[] = { 3f, 3f };
		g2d.setStroke(new BasicStroke(3.0f, BasicStroke.CAP_BUTT,
				BasicStroke.JOIN_MITER, 10.0f, dash1, 0.0f));

		pointIndex = 1;
		g2d.setColor(Color.BLUE);

		while (pointIndex <= lineIndex) {

			if (pointIndex == lineIndex) {
				if (!(arrayListPoints.size() <= pointIndex - 1)) {
					g2d.drawLine( (int) arrayListPoints.get(pointIndex - 1)
							.getY(),(int) arrayListPoints.get(pointIndex - 1)
							.getX(),  (int) tool.getY(),(int) tool.getX());

				}
			} else {
				if (!(arrayListPoints.size() <= pointIndex)) {
					g2d.drawLine( (int) arrayListPoints.get(pointIndex - 1)
							.getY(),(int) arrayListPoints.get(pointIndex - 1)
							.getX(), (int) arrayListPoints.get(pointIndex)
							.getY(), (int) arrayListPoints.get(pointIndex)
							.getX());

				}
			}
			pointIndex++;
		}
		g2d.setColor(Color.BLACK);
		g2d.setStroke(new BasicStroke());
	}

	private void drawWrintings() {
		g2d.rotate(Math.PI / 2);
		// Eixo Y//
		g2d.drawString("_0", 4, -1);
		g2d.drawString("_50", 4, -50);
		g2d.drawString("_100", 4, -100);
		g2d.drawString("_150", 4, -150);
		g2d.drawString("_200", 4, -200);
		g2d.drawString("_250", 4, -250);
		g2d.drawString("_300", 4, -300);
		g2d.drawString("_350", 4, -350);
		g2d.drawString("_400", 4, -400);
		g2d.drawString("_450", 4, -450);
		g2d.drawString("_500", 4, -500);

		// Eixo x//
		g2d.drawString("|0", 1, -4);
		g2d.drawString("|50", 50, -4);
		g2d.drawString("|100", 100, -4);
		g2d.drawString("|150", 150, -4);
		g2d.drawString("|200", 200, -4);
		g2d.drawString("|250", 250, -4);
		g2d.drawString("|300", 300, -4);
		g2d.drawString("|350", 350, -4);
		g2d.drawString("|400", 400, -4);
		g2d.drawString("|450", 450, -4);
		g2d.drawString("|500", 500, -4);

		g2d.rotate(-Math.PI / 2);

	}
	
	private void drawStatus(Boolean drilling) {
		isDrilling = drilling;
		repaint();
	}

	private void drawMesage() {
		g2d.rotate(Math.PI / 2);
		g2d.setColor(Color.RED);
		g2d.setFont(new Font("arial",Font.BOLD,14));
		g2d.drawString("Drilling", (int)tool
						.getY()+ tool.getDiameter()/2,
						-(int)(tool.getX())- tool.getDiameter()/2);
		g2d.setColor(Color.BLACK);
		g2d.rotate(-Math.PI / 2);
		
	}

	// ////////////////////////////////// inner listener class ActionListener
	class TimerAction implements ActionListener {

		// ================================================== actionPerformed
		public void actionPerformed(ActionEvent e) {
			if (!pass)
				makeDelay();

			pass = true;
			tool.setBounds(getWidth(), getHeight());
			System.out.println("XTool:" + tool.getX());
			System.out.println("XPath:" + toolPath.getNextX());
			System.out.println("YTool:" + tool.getY());
			System.out.println("YPath:" + toolPath.getNextY());

			if ((Math.abs(tool.getY() - (int) toolPath.getNextY()) <= 2)
					&& (Math.abs(tool.getX() - (int) toolPath.getNextX()) <= 2)) {
				if (toolPath.isLast()) {
					// TERMINOU A TRAJETORIA
					m_timer.stop();
				}
				else if(lineIndex!=0){
				drawStatus(true);
				pass = false;
				}
				isDrilling = true;
				lineIndex++;
				initialPoint = new Point(toolPath.getNextX(), toolPath
						.getNextY());
				toolPath.nextPoint();
				lastPoint = new Point(toolPath.getNextX(), toolPath.getNextY());
				tool.setNewPointMark();
			}

			if (Math.abs(tool.getY() - (int) toolPath.getNextY()) > 1) {
				tool.moveY(toolPath.getNextX(), toolPath.getNextY());
			}
			if (Math.abs(tool.getX() - (int) toolPath.getNextX()) > 1) {
				tool.moveX(toolPath.getNextX(), toolPath.getNextY());
			}

			if (pass)
				drawStatus(false); // Repaint indirectly calls paintComponent.

		}
	}

	public void makeDelay() {
		if (lineIndex != 0) {
			setAnimation(false);
			try {
				Thread.sleep((int)(50/velocity * 1000));
			} catch (InterruptedException e) {
				e.printStackTrace();
			}
			setAnimation(true);
			isDrilling = false;
		}
	}

}// endclass
